@@ -1,7 +1,7 @@
 /*
  * Copyright 2008-2009 Katholieke Universiteit Leuven
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, K.U.Leuven, Departement
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
@@ -10,8 +10,10 @@
 #ifndef ISL_LP_H
 #define ISL_LP_H
 
-#include <isl/map.h>
-#include <isl/set.h>
+#include <isl/int.h>
+#include <isl/vec.h>
+#include <isl/map_type.h>
+#include <isl/set_type.h>
 
 enum isl_lp_result {
 	isl_lp_error = -1,
