@@ -1,7 +1,7 @@
 /*
  * Copyright 2008-2009 Katholieke Universiteit Leuven
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, K.U.Leuven, Departement
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
@@ -22,6 +22,7 @@ extern "C" {
 
 void isl_seq_clr(isl_int *p, unsigned len);
 void isl_seq_set(isl_int *p, isl_int v, unsigned len);
+void isl_seq_set_si(isl_int *p, int v, unsigned len);
 void isl_seq_neg(isl_int *dat, isl_int *src, unsigned len);
 void isl_seq_cpy(isl_int *dst, isl_int *src, unsigned len);
 void isl_seq_addmul(isl_int *dst, isl_int f, isl_int *src, unsigned len);
@@ -36,6 +37,7 @@ void isl_seq_combine(isl_int *dst, isl_int m1, isl_int *src1,
 			isl_int m2, isl_int *src2, unsigned len);
 void isl_seq_elim(isl_int *dst, isl_int *src, unsigned pos, unsigned len,
 		  isl_int *m);
+void isl_seq_abs_max(isl_int *p, unsigned len, isl_int *max);
 void isl_seq_gcd(isl_int *p, unsigned len, isl_int *gcd);
 void isl_seq_lcm(isl_int *p, unsigned len, isl_int *lcm);
 void isl_seq_normalize(struct isl_ctx *ctx, isl_int *p, unsigned len);
