@@ -1,7 +1,7 @@
 /*
  * Copyright 2008-2009 Katholieke Universiteit Leuven
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, K.U.Leuven, Departement
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
@@ -34,7 +34,7 @@ struct isl_basic_map *isl_basic_map_implicit_equalities(
 	if (bmap->n_ineq <= 1)
 		return bmap;
 
-	tab = isl_tab_from_basic_map(bmap);
+	tab = isl_tab_from_basic_map(bmap, 0);
 	if (isl_tab_detect_implicit_equalities(tab) < 0)
 		goto error;
 	bmap = isl_basic_map_update_from_tab(bmap, tab);
@@ -353,6 +353,72 @@ error:
 	return NULL;
 }
 
+/* Move "sample" to a point that is one up (or down) from the original
+ * point in dimension "pos".
+ */
+static void adjacent_point(__isl_keep isl_vec *sample, int pos, int up)
+{
+	if (up)
+		isl_int_add_ui(sample->el[1 + pos], sample->el[1 + pos], 1);
+	else
+		isl_int_sub_ui(sample->el[1 + pos], sample->el[1 + pos], 1);
+}
+
+/* Check if any points that are adjacent to "sample" also belong to "bset".
+ * If so, add them to "hull" and return the updated hull.
+ *
+ * Before checking whether and adjacent point belongs to "bset", we first
+ * check whether it already belongs to "hull" as this test is typically
+ * much cheaper.
+ */
+static __isl_give isl_basic_set *add_adjacent_points(
+	__isl_take isl_basic_set *hull, __isl_take isl_vec *sample,
+	__isl_keep isl_basic_set *bset)
+{
+	int i, up;
+	int dim;
+
+	if (!sample)
+		goto error;
+
+	dim = isl_basic_set_dim(hull, isl_dim_set);
+
+	for (i = 0; i < dim; ++i) {
+		for (up = 0; up <= 1; ++up) {
+			int contains;
+			isl_basic_set *point;
+
+			adjacent_point(sample, i, up);
+			contains = isl_basic_set_contains(hull, sample);
+			if (contains < 0)
+				goto error;
+			if (contains) {
+				adjacent_point(sample, i, !up);
+				continue;
+			}
+			contains = isl_basic_set_contains(bset, sample);
+			if (contains < 0)
+				goto error;
+			if (contains) {
+				point = isl_basic_set_from_vec(
+							isl_vec_copy(sample));
+				hull = affine_hull(hull, point);
+			}
+			adjacent_point(sample, i, !up);
+			if (contains)
+				break;
+		}
+	}
+
+	isl_vec_free(sample);
+
+	return hull;
+error:
+	isl_vec_free(sample);
+	isl_basic_set_free(hull);
+	return NULL;
+}
+
 /* Extend an initial (under-)approximation of the affine hull of basic
  * set represented by the tableau "tab"
  * by looking for points that do not satisfy one of the equalities
@@ -361,9 +427,13 @@ error:
  *
  * The caller of this function ensures that "tab" is bounded or
  * that tab->basis and tab->n_unbounded have been set appropriately.
+ *
+ * "bset" may be either NULL or the basic set represented by "tab".
+ * If "bset" is not NULL, we check for any point we find if any
+ * of its adjacent points also belong to "bset".
  */
-static struct isl_basic_set *extend_affine_hull(struct isl_tab *tab,
-	struct isl_basic_set *hull)
+static __isl_give isl_basic_set *extend_affine_hull(struct isl_tab *tab,
+	__isl_take isl_basic_set *hull, __isl_keep isl_basic_set *bset)
 {
 	int i, j;
 	unsigned dim;
@@ -402,6 +472,9 @@ static struct isl_basic_set *extend_affine_hull(struct isl_tab *tab,
 			tab = isl_tab_add_sample(tab, isl_vec_copy(sample));
 		if (!tab)
 			goto error;
+		if (bset)
+			hull = add_adjacent_points(hull, isl_vec_copy(sample),
+						    bset);
 		point = isl_basic_set_from_vec(sample);
 		hull = affine_hull(hull, point);
 		if (!hull)
@@ -414,35 +487,186 @@ error:
 	return NULL;
 }
 
+/* Drop all constraints in bmap that involve any of the dimensions
+ * first to first+n-1.
+ */
+static __isl_give isl_basic_map *isl_basic_map_drop_constraints_involving(
+	__isl_take isl_basic_map *bmap, unsigned first, unsigned n)
+{
+	int i;
+
+	if (n == 0)
+		return bmap;
+
+	bmap = isl_basic_map_cow(bmap);
+
+	if (!bmap)
+		return NULL;
+
+	for (i = bmap->n_eq - 1; i >= 0; --i) {
+		if (isl_seq_first_non_zero(bmap->eq[i] + 1 + first, n) == -1)
+			continue;
+		isl_basic_map_drop_equality(bmap, i);
+	}
+
+	for (i = bmap->n_ineq - 1; i >= 0; --i) {
+		if (isl_seq_first_non_zero(bmap->ineq[i] + 1 + first, n) == -1)
+			continue;
+		isl_basic_map_drop_inequality(bmap, i);
+	}
+
+	return bmap;
+}
+
 /* Drop all constraints in bset that involve any of the dimensions
  * first to first+n-1.
  */
 __isl_give isl_basic_set *isl_basic_set_drop_constraints_involving(
 	__isl_take isl_basic_set *bset, unsigned first, unsigned n)
+{
+	return isl_basic_map_drop_constraints_involving(bset, first, n);
+}
+
+/* Drop all constraints in bmap that do not involve any of the dimensions
+ * first to first + n - 1 of the given type.
+ */
+__isl_give isl_basic_map *isl_basic_map_drop_constraints_not_involving_dims(
+	__isl_take isl_basic_map *bmap,
+	enum isl_dim_type type, unsigned first, unsigned n)
 {
 	int i;
+	unsigned dim;
 
 	if (n == 0)
-		return bset;
-
-	bset = isl_basic_set_cow(bset);
-
-	if (!bset)
+		return isl_basic_map_set_to_empty(bmap);
+	bmap = isl_basic_map_cow(bmap);
+	if (!bmap)
 		return NULL;
 
-	for (i = bset->n_eq - 1; i >= 0; --i) {
-		if (isl_seq_first_non_zero(bset->eq[i] + 1 + first, n) == -1)
+	dim = isl_basic_map_dim(bmap, type);
+	if (first + n > dim || first + n < first)
+		isl_die(isl_basic_map_get_ctx(bmap), isl_error_invalid,
+			"index out of bounds", return isl_basic_map_free(bmap));
+
+	first += isl_basic_map_offset(bmap, type) - 1;
+
+	for (i = bmap->n_eq - 1; i >= 0; --i) {
+		if (isl_seq_first_non_zero(bmap->eq[i] + 1 + first, n) != -1)
 			continue;
-		isl_basic_set_drop_equality(bset, i);
+		isl_basic_map_drop_equality(bmap, i);
 	}
 
-	for (i = bset->n_ineq - 1; i >= 0; --i) {
-		if (isl_seq_first_non_zero(bset->ineq[i] + 1 + first, n) == -1)
+	for (i = bmap->n_ineq - 1; i >= 0; --i) {
+		if (isl_seq_first_non_zero(bmap->ineq[i] + 1 + first, n) != -1)
 			continue;
-		isl_basic_set_drop_inequality(bset, i);
+		isl_basic_map_drop_inequality(bmap, i);
 	}
 
-	return bset;
+	return bmap;
+}
+
+/* Drop all constraints in bset that do not involve any of the dimensions
+ * first to first + n - 1 of the given type.
+ */
+__isl_give isl_basic_set *isl_basic_set_drop_constraints_not_involving_dims(
+	__isl_take isl_basic_set *bset,
+	enum isl_dim_type type, unsigned first, unsigned n)
+{
+	return isl_basic_map_drop_constraints_not_involving_dims(bset,
+							    type, first, n);
+}
+
+/* Drop all constraints in bmap that involve any of the dimensions
+ * first to first + n - 1 of the given type.
+ */
+__isl_give isl_basic_map *isl_basic_map_drop_constraints_involving_dims(
+	__isl_take isl_basic_map *bmap,
+	enum isl_dim_type type, unsigned first, unsigned n)
+{
+	unsigned dim;
+
+	if (!bmap)
+		return NULL;
+	if (n == 0)
+		return bmap;
+
+	dim = isl_basic_map_dim(bmap, type);
+	if (first + n > dim || first + n < first)
+		isl_die(isl_basic_map_get_ctx(bmap), isl_error_invalid,
+			"index out of bounds", return isl_basic_map_free(bmap));
+
+	bmap = isl_basic_map_remove_divs_involving_dims(bmap, type, first, n);
+	first += isl_basic_map_offset(bmap, type) - 1;
+	return isl_basic_map_drop_constraints_involving(bmap, first, n);
+}
+
+/* Drop all constraints in bset that involve any of the dimensions
+ * first to first + n - 1 of the given type.
+ */
+__isl_give isl_basic_set *isl_basic_set_drop_constraints_involving_dims(
+	__isl_take isl_basic_set *bset,
+	enum isl_dim_type type, unsigned first, unsigned n)
+{
+	return isl_basic_map_drop_constraints_involving_dims(bset,
+							    type, first, n);
+}
+
+/* Drop all constraints in map that involve any of the dimensions
+ * first to first + n - 1 of the given type.
+ */
+__isl_give isl_map *isl_map_drop_constraints_involving_dims(
+	__isl_take isl_map *map,
+	enum isl_dim_type type, unsigned first, unsigned n)
+{
+	int i;
+	unsigned dim;
+
+	if (!map)
+		return NULL;
+	if (n == 0)
+		return map;
+
+	dim = isl_map_dim(map, type);
+	if (first + n > dim || first + n < first)
+		isl_die(isl_map_get_ctx(map), isl_error_invalid,
+			"index out of bounds", return isl_map_free(map));
+
+	map = isl_map_cow(map);
+	if (!map)
+		return NULL;
+
+	for (i = 0; i < map->n; ++i) {
+		map->p[i] = isl_basic_map_drop_constraints_involving_dims(
+						    map->p[i], type, first, n);
+		if (!map->p[i])
+			return isl_map_free(map);
+	}
+
+	return map;
+}
+
+/* Drop all constraints in set that involve any of the dimensions
+ * first to first + n - 1 of the given type.
+ */
+__isl_give isl_set *isl_set_drop_constraints_involving_dims(
+	__isl_take isl_set *set,
+	enum isl_dim_type type, unsigned first, unsigned n)
+{
+	return isl_map_drop_constraints_involving_dims(set, type, first, n);
+}
+
+/* Construct an initial underapproximatino of the hull of "bset"
+ * from "sample" and any of its adjacent points that also belong to "bset".
+ */
+static __isl_give isl_basic_set *initialize_hull(__isl_keep isl_basic_set *bset,
+	__isl_take isl_vec *sample)
+{
+	isl_basic_set *hull;
+
+	hull = isl_basic_set_from_vec(isl_vec_copy(sample));
+	hull = add_adjacent_points(hull, sample, bset);
+
+	return hull;
 }
 
 /* Look for all equalities satisfied by the integer points in bset,
@@ -480,7 +704,7 @@ static struct isl_basic_set *uset_affine_hull_bounded(struct isl_basic_set *bset
 		}
 	}
 
-	tab = isl_tab_from_basic_set(bset);
+	tab = isl_tab_from_basic_set(bset, 1);
 	if (!tab)
 		goto error;
 	if (tab->empty) {
@@ -488,8 +712,6 @@ static struct isl_basic_set *uset_affine_hull_bounded(struct isl_basic_set *bset
 		isl_vec_free(sample);
 		return isl_basic_set_set_to_empty(bset);
 	}
-	if (isl_tab_track_bset(tab, isl_basic_set_copy(bset)) < 0)
-		goto error;
 
 	if (!sample) {
 		struct isl_tab_undo *snap;
@@ -509,10 +731,10 @@ static struct isl_basic_set *uset_affine_hull_bounded(struct isl_basic_set *bset
 		return isl_basic_set_set_to_empty(bset);
 	}
 
-	hull = isl_basic_set_from_vec(sample);
+	hull = initialize_hull(bset, sample);
 
+	hull = extend_affine_hull(tab, hull, bset);
 	isl_basic_set_free(bset);
-	hull = extend_affine_hull(tab, hull);
 	isl_tab_free(tab);
 
 	return hull;
@@ -583,7 +805,7 @@ struct isl_tab *isl_tab_detect_equalities(struct isl_tab *tab,
 {
 	int j;
 	struct isl_vec *sample;
-	struct isl_basic_set *hull;
+	struct isl_basic_set *hull = NULL;
 	struct isl_tab_undo *snap;
 
 	if (!tab || !tab_cone)
@@ -624,7 +846,7 @@ struct isl_tab *isl_tab_detect_equalities(struct isl_tab *tab,
 
 	isl_vec_free(sample);
 
-	hull = extend_affine_hull(tab, hull);
+	hull = extend_affine_hull(tab, hull, NULL);
 	if (!hull)
 		goto error;
 
@@ -648,6 +870,7 @@ struct isl_tab *isl_tab_detect_equalities(struct isl_tab *tab,
 
 	return tab;
 error:
+	isl_basic_set_free(hull);
 	isl_tab_free(tab);
 	return NULL;
 }
@@ -718,9 +941,10 @@ static struct isl_basic_set *affine_hull_with_cone(struct isl_basic_set *bset,
 
 	hull = uset_affine_hull_bounded(bset);
 
-	if (!hull)
+	if (!hull) {
+		isl_mat_free(Q);
 		isl_mat_free(U);
-	else {
+	} else {
 		struct isl_vec *sample = isl_vec_copy(hull->sample);
 		U = isl_mat_drop_cols(U, 1 + total - cone_dim, cone_dim);
 		if (sample && sample->size > 0)
@@ -840,6 +1064,7 @@ static struct isl_basic_set *equalities_in_underlying_set(
 
 	return hull;
 error:
+	isl_mat_free(T1);
 	isl_mat_free(T2);
 	isl_basic_set_free(bset);
 	isl_basic_set_free(hull);
@@ -873,7 +1098,7 @@ struct isl_basic_map *isl_basic_map_detect_equalities(
 		isl_basic_set_free(hull);
 		return isl_basic_map_set_to_empty(bmap);
 	}
-	bmap = isl_basic_map_extend_dim(bmap, isl_dim_copy(bmap->dim), 0,
+	bmap = isl_basic_map_extend_space(bmap, isl_space_copy(bmap->dim), 0,
 					hull->n_eq, 0);
 	for (i = 0; i < hull->n_eq; ++i) {
 		j = isl_basic_map_alloc_equality(bmap);
