@@ -1,7 +1,7 @@
 /*
  * Copyright 2010      INRIA Saclay
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, INRIA Saclay - Ile-de-France,
  * Parc Club Orsay Universite, ZAC des vignes, 4 rue Jacques Monod,
@@ -70,12 +70,12 @@ error:
 }
 
 #ifdef HAS_TYPE
-__isl_give UNION *FN(UNION,zero)(__isl_take isl_space *dim, enum isl_fold type)
+__isl_give UNION *FN(UNION,ZERO)(__isl_take isl_space *dim, enum isl_fold type)
 {
 	return FN(UNION,alloc)(dim, type, 16);
 }
 #else
-__isl_give UNION *FN(UNION,zero)(__isl_take isl_space *dim)
+__isl_give UNION *FN(UNION,ZERO)(__isl_take isl_space *dim)
 {
 	return FN(UNION,alloc)(dim, 16);
 }
@@ -138,9 +138,9 @@ __isl_give PART *FN(FN(UNION,extract),PARTS)(__isl_keep UNION *u,
 				    &has_dim, dim, 0);
 	if (!entry)
 #ifdef HAS_TYPE
-		return FN(PART,zero)(dim, u->type);
+		return FN(PART,ZERO)(dim, u->type);
 #else
-		return FN(PART,zero)(dim);
+		return FN(PART,ZERO)(dim);
 #endif
 	isl_space_free(dim);
 	return FN(PART,copy)(entry->data);
@@ -158,7 +158,7 @@ __isl_give UNION *FN(FN(UNION,add),PARTS)(__isl_take UNION *u,
 	if (!part)
 		goto error;
 
-	if (FN(PART,is_zero)(part)) {
+	if (DEFAULT_IS_ZERO && FN(PART,IS_ZERO)(part)) {
 		FN(PART,free)(part);
 		return u;
 	}
@@ -184,7 +184,7 @@ __isl_give UNION *FN(FN(UNION,add),PARTS)(__isl_take UNION *u,
 		if (!entry->data)
 			goto error;
 		FN(PART,free)(part);
-		if (FN(PART,is_zero)(entry->data)) {
+		if (DEFAULT_IS_ZERO && FN(PART,IS_ZERO)(entry->data)) {
 			FN(PART,free)(entry->data);
 			isl_hash_table_remove(u->dim->ctx, &u->table, entry);
 		}
@@ -214,9 +214,9 @@ __isl_give UNION *FN(UNION,dup)(__isl_keep UNION *u)
 		return NULL;
 
 #ifdef HAS_TYPE
-	dup = FN(UNION,zero)(isl_space_copy(u->dim), u->type);
+	dup = FN(UNION,ZERO)(isl_space_copy(u->dim), u->type);
 #else
-	dup = FN(UNION,zero)(isl_space_copy(u->dim));
+	dup = FN(UNION,ZERO)(isl_space_copy(u->dim));
 #endif
 	if (FN(FN(UNION,foreach),PARTS)(u, &add_part, &dup) < 0)
 		goto error;
@@ -244,18 +244,19 @@ static int free_u_entry(void **entry, void *user)
 	return 0;
 }
 
-void FN(UNION,free)(__isl_take UNION *u)
+void *FN(UNION,free)(__isl_take UNION *u)
 {
 	if (!u)
-		return;
+		return NULL;
 
 	if (--u->ref > 0)
-		return;
+		return NULL;
 
 	isl_hash_table_foreach(u->dim->ctx, &u->table, &free_u_entry, NULL);
 	isl_hash_table_clear(&u->table);
 	isl_space_free(u->dim);
 	free(u);
+	return NULL;
 }
 
 S(UNION,align) {
@@ -365,9 +366,9 @@ __isl_give UNION *FN(FN(UNION,from),PARTS)(__isl_take PART *part)
 	dim = isl_space_drop_dims(dim, isl_dim_in, 0, isl_space_dim(dim, isl_dim_in));
 	dim = isl_space_drop_dims(dim, isl_dim_out, 0, isl_space_dim(dim, isl_dim_out));
 #ifdef HAS_TYPE
-	u = FN(UNION,zero)(dim, part->type);
+	u = FN(UNION,ZERO)(dim, part->type);
 #else
-	u = FN(UNION,zero)(dim);
+	u = FN(UNION,ZERO)(dim);
 #endif
 	u = FN(FN(UNION,add),PARTS)(u, part);
 
@@ -377,18 +378,66 @@ __isl_give UNION *FN(FN(UNION,from),PARTS)(__isl_take PART *part)
 S(UNION,match_bin_data) {
 	UNION *u2;
 	UNION *res;
+	__isl_give PART *(*fn)(__isl_take PART *, __isl_take PART *);
 };
 
+/* Check if data->u2 has an element living in the same space as *entry.
+ * If so, call data->fn on the two elements and add the result to
+ * data->res.
+ */
+static int match_bin_entry(void **entry, void *user)
+{
+	S(UNION,match_bin_data) *data = user;
+	uint32_t hash;
+	struct isl_hash_table_entry *entry2;
+	isl_space *space;
+	PART *part = *entry;
+
+	space = FN(PART,get_space)(part);
+	hash = isl_space_get_hash(space);
+	entry2 = isl_hash_table_find(data->u2->dim->ctx, &data->u2->table,
+				     hash, &has_dim, space, 0);
+	isl_space_free(space);
+	if (!entry2)
+		return 0;
+
+	part = FN(PART, copy)(part);
+	part = data->fn(part, FN(PART, copy)(entry2->data));
+
+	if (DEFAULT_IS_ZERO) {
+		int empty;
+
+		empty = FN(PART,IS_ZERO)(part);
+		if (empty < 0) {
+			FN(PART,free)(part);
+			return -1;
+		}
+		if (empty) {
+			FN(PART,free)(part);
+			return 0;
+		}
+	}
+
+	data->res = FN(FN(UNION,add),PARTS)(data->res, part);
+
+	return 0;
+}
+
 /* This function is currently only used from isl_polynomial.c
  * and not from isl_fold.c.
  */
 static __isl_give UNION *match_bin_op(__isl_take UNION *u1,
 	__isl_take UNION *u2,
-	int (*fn)(void **, void *)) __attribute__ ((unused));
+	__isl_give PART *(*fn)(__isl_take PART *, __isl_take PART *))
+	__attribute__ ((unused));
+/* For each pair of elements in "u1" and "u2" living in the same space,
+ * call "fn" and collect the results.
+ */
 static __isl_give UNION *match_bin_op(__isl_take UNION *u1,
-	__isl_take UNION *u2, int (*fn)(void **, void *))
+	__isl_take UNION *u2,
+	__isl_give PART *(*fn)(__isl_take PART *, __isl_take PART *))
 {
-	S(UNION,match_bin_data) data = { NULL, NULL };
+	S(UNION,match_bin_data) data = { NULL, NULL, fn };
 
 	u1 = FN(UNION,align_params)(u1, FN(UNION,get_space)(u2));
 	u2 = FN(UNION,align_params)(u2, FN(UNION,get_space)(u1));
@@ -402,7 +451,8 @@ static __isl_give UNION *match_bin_op(__isl_take UNION *u1,
 #else
 	data.res = FN(UNION,alloc)(isl_space_copy(u1->dim), u1->table.n);
 #endif
-	if (isl_hash_table_foreach(u1->dim->ctx, &u1->table, fn, &data) < 0)
+	if (isl_hash_table_foreach(u1->dim->ctx, &u1->table,
+				    &match_bin_entry, &data) < 0)
 		goto error;
 
 	FN(UNION,free)(u1);
@@ -425,19 +475,22 @@ static int any_set_entry(void **entry, void *user)
 {
 	S(UNION,any_set_data) *data = user;
 	PW *pw = *entry;
-	int empty;
 
 	pw = FN(PW,copy)(pw);
 	pw = data->fn(pw, isl_set_copy(data->set));
 
-	empty = FN(PW,is_zero)(pw);
-	if (empty < 0) {
-		FN(PW,free)(pw);
-		return -1;
-	}
-	if (empty) {
-		FN(PW,free)(pw);
-		return 0;
+	if (DEFAULT_IS_ZERO) {
+		int empty;
+
+		empty = FN(PW,IS_ZERO)(pw);
+		if (empty < 0) {
+			FN(PW,free)(pw);
+			return -1;
+		}
+		if (empty) {
+			FN(PW,free)(pw);
+			return 0;
+		}
 	}
 
 	data->res = FN(FN(UNION,add),PARTS)(data->res, pw);
@@ -521,7 +574,6 @@ static int match_domain_entry(void **entry, void *user)
 	struct isl_hash_table_entry *entry2;
 	PW *pw = *entry;
 	isl_space *space;
-	int empty;
 
 	space = FN(PW,get_domain_space)(pw);
 	hash = isl_space_get_hash(space);
@@ -534,14 +586,18 @@ static int match_domain_entry(void **entry, void *user)
 	pw = FN(PW,copy)(pw);
 	pw = data->fn(pw, isl_set_copy(entry2->data));
 
-	empty = FN(PW,is_zero)(pw);
-	if (empty < 0) {
-		FN(PW,free)(pw);
-		return -1;
-	}
-	if (empty) {
-		FN(PW,free)(pw);
-		return 0;
+	if (DEFAULT_IS_ZERO) {
+		int empty;
+
+		empty = FN(PW,IS_ZERO)(pw);
+		if (empty < 0) {
+			FN(PW,free)(pw);
+			return -1;
+		}
+		if (empty) {
+			FN(PW,free)(pw);
+			return 0;
+		}
 	}
 
 	data->res = FN(FN(UNION,add),PARTS)(data->res, pw);
@@ -606,6 +662,7 @@ __isl_give UNION *FN(UNION,gist)(__isl_take UNION *u,
 	return match_domain_op(u, uset, &FN(PW,gist));
 }
 
+#ifndef NO_EVAL
 __isl_give isl_qpolynomial *FN(UNION,eval)(__isl_take UNION *u,
 	__isl_take isl_point *pnt)
 {
@@ -639,6 +696,7 @@ error:
 	isl_point_free(pnt);
 	return NULL;
 }
+#endif
 
 static int coalesce_entry(void **entry, void *user)
 {
@@ -709,13 +767,13 @@ __isl_give UNION *FN(UNION,mul_isl_int)(__isl_take UNION *u, isl_int v)
 	if (isl_int_is_one(v))
 		return u;
 
-	if (u && isl_int_is_zero(v)) {
+	if (DEFAULT_IS_ZERO && u && isl_int_is_zero(v)) {
 		UNION *zero;
 		isl_space *dim = FN(UNION,get_space)(u);
 #ifdef HAS_TYPE
-		zero = FN(UNION,zero)(dim, u->type);
+		zero = FN(UNION,ZERO)(dim, u->type);
 #else
-		zero = FN(UNION,zero)(dim);
+		zero = FN(UNION,ZERO)(dim);
 #endif
 		FN(UNION,free)(u);
 		return zero;
