@@ -14,7 +14,7 @@
 #include <osl/relation.h>
 #endif
 
-CloogDomain *cloog_domain_from_isl_set(struct isl_set *set)
+CloogDomain *cloog_domain_from_isl_set(__isl_take isl_set *set)
 {
 	if (isl_set_is_params(set))
 		set = isl_set_from_params(set);
@@ -28,7 +28,7 @@ __isl_give isl_set *isl_set_from_cloog_domain(CloogDomain *domain)
 	return (isl_set *)domain;
 }
 
-CloogScattering *cloog_scattering_from_isl_map(struct isl_map *map)
+CloogScattering *cloog_scattering_from_isl_map(__isl_take isl_map *map)
 {
 	return (CloogScattering *)map;
 }
@@ -310,7 +310,8 @@ int cloog_domain_follows(CloogDomain *dom1, CloogDomain *dom2, unsigned level)
 CloogDomain *cloog_domain_empty(CloogDomain *template)
 {
 	isl_set *set = isl_set_from_cloog_domain(template);
-	return cloog_domain_from_isl_set(isl_set_empty_like(set));
+	isl_space *space = isl_set_get_space(set);
+	return cloog_domain_from_isl_set(isl_set_empty(space));
 }
 
 
@@ -1010,7 +1011,7 @@ static int basic_set_stride_lower(__isl_take isl_basic_set *bset, void *user)
 	struct cloog_stride_lower *csl = (struct cloog_stride_lower *)user;
 	int r;
 
-	csl->bounds = isl_basic_set_universe_like(bset);
+	csl->bounds = isl_basic_set_universe(isl_basic_set_get_space(bset));
 	if (csl->stride->constraint)
 		r = isl_basic_set_foreach_constraint(bset,
 					&constraint_stride_lower_c, csl);
@@ -1037,7 +1038,7 @@ CloogDomain *cloog_domain_stride_lower_bound(CloogDomain *domain, int level,
 
 	csl.stride = stride;
 	csl.level = level;
-	csl.set = isl_set_empty_like(set);
+	csl.set = isl_set_empty(isl_set_get_space(set));
 
 	r = isl_set_foreach_basic_set(set, basic_set_stride_lower, &csl);
 	assert(r == 0);
@@ -1146,7 +1147,7 @@ CloogDomain *cloog_domain_bound_splitter(CloogDomain *dom, int level)
 	isl_set *set = isl_set_from_cloog_domain(dom);
 	int r;
 	cbs.level = level;
-	cbs.set = isl_set_universe_like(set);
+	cbs.set = isl_set_universe(isl_set_get_space(set));
 	r = isl_set_foreach_basic_set(set, basic_set_bound_split, &cbs);
 	assert(r == 0);
 	return cloog_domain_from_isl_set(cbs.set);
@@ -1695,6 +1696,11 @@ static int find_stride(__isl_take isl_constraint *c, void *user)
 	int n;
 	isl_int v;
 
+	if (!isl_constraint_is_equality(c)) {
+		isl_constraint_free(c);
+		return 0;
+	}
+
 	data = (struct cloog_isl_find_stride_data *)user;
 
 	if (data->stride) {
